﻿using Sc.Integration.Ref.App.Models;
using Sitecore.Data;
using Sitecore.Data.Items;
using Sitecore.SecurityModel;
using Sitecore.Services.Core;
using Sitecore.Services.Infrastructure.Web.Http;
using Sitecore.Web.Http.Filters;
using System.Net;
using System.Net.Http;
using System.Web.Mvc;

namespace Sc.Integration.Ref.App.Controllers
{
    [ServicesController("scirefapp/sampleitems")]
    [Authorize]
    public class SampleItemsController : ServicesApiController
    {
        private static readonly ID SampleItemTemplateId = new ID("{EE2EBE61-0948-49EB-B501-F1C6D346F3E7}");

        [HttpGet]
        public System.Web.Http.IHttpActionResult Get(string id)
        {
            if (string.IsNullOrEmpty(id))
            {
                return BadRequest("Root item ID is not specified");
            }

            ID itemId;
            if (!ID.TryParse(id, out itemId) || ID.IsNullOrEmpty(itemId))
            {
                return BadRequest("Root item ID is not valid");
            }

            var db = Sitecore.Configuration.Factory.GetDatabase("core");
            Item item;
            using (new SecurityDisabler())
            {
                item = db?.GetItem(itemId);
                if (item == null)
                {
                    return NotFound();
                }

                if (item.TemplateID != SampleItemTemplateId)
                {
                    return ResponseMessage(Request.CreateErrorResponse(HttpStatusCode.Forbidden,
                        "Item has the wrong template type"));
                }
            }

            var result = new SampleItem
            {
                Name = item.Name,
                Description = item["Description"],
                FieldA = item["FieldA"],
                FieldB = item["FieldB"]
            };
            return Ok(result);
        }

        [HttpPost]
        [ValidateHttpAntiForgeryToken]  // This attribute secures the method with Anti-CSRF
        public System.Web.Http.IHttpActionResult Edit(string id, SampleItem updatedItem)
        {
            if (string.IsNullOrEmpty(id))
            {
                return BadRequest("Root item ID is not specified");
            }

            if (updatedItem == null)
            {
                return BadRequest("UpdatedItem is not specified");
            }

            ID itemId;
            if (!ID.TryParse(id, out itemId) || ID.IsNullOrEmpty(itemId))
            {
                return BadRequest("Root item ID is not valid");
            }

            var db = Sitecore.Configuration.Factory.GetDatabase("core");
            using (new SecurityDisabler())
            {
                var item = db?.GetItem(itemId);
                if (item == null)
                {
                    return NotFound();
                }

                if (item.TemplateID != SampleItemTemplateId)
                {
                    return ResponseMessage(Request.CreateErrorResponse(HttpStatusCode.Forbidden,
                        "Item has the wrong template type"));
                }

                item.Editing.BeginEdit();
                try
                {
                    item.Name = updatedItem.Name;
                    item["Description"] = updatedItem.Description;
                    item["FieldA"] = updatedItem.FieldA;
                    item["FieldB"] = updatedItem.FieldB;
                }
                finally
                {
                    item.Editing.EndEdit();
                }
            }
            return ResponseMessage(Request.CreateResponse(HttpStatusCode.NoContent));
        }
    }
}